--[[============================================================================
masking_class.lua
============================================================================]]--

-- Placeholder for the dialog
local dialog = nil

-- Placeholder to expose the ViewBuilder outside the show_dialog() function
local vb = renoise.ViewBuilder()
local mask_dialog = nil
local dialog_content = nil
mask_file = nil
local mask_table = {
  "Note", "Sign","Octave","Instrument","Volume", "Panning","Delay",
  "EffectNumber","EffectValue", "Automation"
}

-- Reload the script whenever this file is saved. 
-- Additionally, execute the attached function.
_AUTO_RELOAD_DEBUG = function()
 
end
MASK_SELECTED_INVERTED = 1
MASK_ALL_INVERTED = 2
MASK_ALL_TRUE = 3
MASK_ALL_FALSE = 4

local SEP = "/"

if (os.platform() == "WINDOWS") then
  SEP = "\\"
end

function get_tools_root()    
  local dir = renoise.tool().bundle_path
  return dir:sub(1,dir:find("Tools")+5)      
end

mask_file = get_tools_root().."masks.xml"

-- Read from the manifest.xml file.
class "MaskingTable" (renoise.Document.DocumentNode)
  function MaskingTable:__init()    
    renoise.Document.DocumentNode.__init(self) 
    for x = 1,#mask_table do
      self:add_property(mask_table[x], "true")
    end
    self:add_property("NoteMaskSelection", "1")
    self:add_property("EffectMaskSelection", "1")
  end

local masking_table = MaskingTable()
local ok,err = masking_table:load_from(mask_file)

--------------------------------------------------------------------------------
-- Main functions
--------------------------------------------------------------------------------

local function mask_columns(option)
  local note_column = renoise.song().selected_note_column_index
  local fx_column = renoise.song().selected_effect_column_index  

  if mask_dialog == nil then
    return
  end

  if option == MASK_SELECTED_INVERTED then

    if note_column > 0 then

      if masking_table:property("NoteMaskSelection").value == "1" then
        vb.views.Note.value = not vb.views.Note.value
        masking_table:property("Note").value = tostring(vb.views.Note.value)
      end
      if masking_table:property("NoteMaskSelection").value == "2" then
        vb.views.Sign.value = not vb.views.Sign.value
        masking_table:property("Sign").value = tostring(vb.views.Sign.value)
      end
      if masking_table:property("NoteMaskSelection").value == "3" then
        vb.views.Octave.value = not vb.views.Octave.value
        masking_table:property("Octave").value = tostring(vb.views.Octave.value)
      end
      if masking_table:property("NoteMaskSelection").value == "4" then
        vb.views.Instrument.value = not vb.views.Instrument.value
        masking_table:property("Instrument").value = tostring(vb.views.Instrument.value)
      end
      if masking_table:property("NoteMaskSelection").value == "5" then
        vb.views.Volume.value = not vb.views.Volume.value
        masking_table:property("Volume").value = tostring(vb.views.Volume.value)
      end
      if masking_table:property("NoteMaskSelection").value == "6" then
        vb.views.Panning.value = not vb.views.Panning.value
        masking_table:property("Panning").value = tostring(vb.views.Panning.value)
      end
      if masking_table:property("NoteMaskSelection").value == "7" then
        vb.views.Delay.value = not vb.views.Delay.value
        masking_table:property("Delay").value = tostring(vb.views.Delay.value)
      end


      local nms = tonumber(masking_table:property("NoteMaskSelection").value)

      if nms < 7 then
        masking_table:property("NoteMaskSelection").value = tostring(nms+1)
      else
        masking_table:property("NoteMaskSelection").value = "1"
      end
        
    else

      if masking_table:property("EffectMaskSelection").value == "1" then
        vb.views.EffectNumber.value = not vb.views.EffectNumber.value
        masking_table:property("EffectNumber").value = tostring(vb.views.EffectNumber.value)
      end
      if masking_table:property("EffectMaskSelection").value == "2" then
        vb.views.EffectValue.value = not vb.views.EffectValue.value
        masking_table:property("EffectValue").value = tostring(vb.views.EffectValue.value)
      end
      if masking_table:property("EffectMaskSelection").value == "3" then
        vb.views.Automation.value = not vb.views.Automation.value
        masking_table:property("Automation").value = tostring(vb.views.Automation.value)
      end
      local ems = tonumber(masking_table:property("EffectMaskSelection").value)

      if ems < 3 then
        masking_table:property("EffectMaskSelection").value = tostring(ems+1)
      else
        masking_table:property("EffectMaskSelection").value = "1"
      end

    end

  end

  if option == MASK_ALL_INVERTED then
    if note_column > 0 then
      vb.views.Note.value = not vb.views.Note.value
      vb.views.Sign.value = not vb.views.Sign.value
      vb.views.Octave.value = not vb.views.Octave.value
      vb.views.Instrument.value = not vb.views.Instrument.value
      vb.views.Volume.value = not vb.views.Volume.value
      vb.views.Panning.value = not vb.views.Panning.value
      vb.views.Delay.value = not vb.views.Delay.value
      masking_table:property("Note").value = tostring(vb.views.Note.value)
      masking_table:property("Sign").value = tostring(vb.views.Sign.value)
      masking_table:property("Octave").value = tostring(vb.views.Octave.value)
      masking_table:property("Instrument").value = tostring(vb.views.Instrument.value)
      masking_table:property("Volume").value = tostring(vb.views.Volume.value)
      masking_table:property("Panning").value = tostring(vb.views.Panning.value)
      masking_table:property("Delay").value = tostring(vb.views.Delay.value)
    else
      vb.views.EffectNumber.value = not vb.views.EffectNumber.value
      masking_table:property("EffectNumber").value = tostring(vb.views.EffectNumber.value)
      vb.views.EffectValue.value = not vb.views.EffectValue.value
      masking_table:property("EffectValue").value = tostring(vb.views.EffectValue.value)
      vb.views.Automation.value = not vb.views.Automation.value
      masking_table:property("Automation").value = tostring(vb.views.Automation.value)
    end

    masking_table:property("NoteMaskSelection").value = "1"
    masking_table:property("EffectMaskSelection").value = "1"
  end
    
  if option == MASK_ALL_TRUE then
    if note_column > 0 then
      vb.views.Note.value = true
      vb.views.Sign.value = true
      vb.views.Octave.value = true
      vb.views.Instrument.value = true
      vb.views.Volume.value = true
      vb.views.Panning.value = true
      vb.views.Delay.value = true
      masking_table:property("Note").value = tostring(vb.views.Note.value)
      masking_table:property("Sign").value = tostring(vb.views.Sign.value)
      masking_table:property("Octave").value = tostring(vb.views.Octave.value)
      masking_table:property("Instrument").value = tostring(vb.views.Instrument.value)
      masking_table:property("Volume").value = tostring(vb.views.Volume.value)
      masking_table:property("Panning").value = tostring(vb.views.Panning.value)
      masking_table:property("Delay").value = tostring(vb.views.Delay.value)
    else
      vb.views.EffectNumber.value = true
      masking_table:property("EffectNumber").value = tostring(vb.views.EffectNumber.value)
      vb.views.EffectValue.value = true
      masking_table:property("EffectValue").value = tostring(vb.views.EffectValue.value)
      vb.views.Automation.value = true
      masking_table:property("Automation").value = tostring(vb.views.Automation.value)
    end

    masking_table:property("NoteMaskSelection").value = "1"
    masking_table:property("EffectMaskSelection").value = "1"
  end

  if option == MASK_ALL_FALSE then
    if note_column > 0 then
      vb.views.Note.value = false
      vb.views.Sign.value = false
      vb.views.Octave.value = false
      vb.views.Instrument.value = false
      vb.views.Volume.value = false
      vb.views.Panning.value = false
      vb.views.Delay.value = false
      masking_table:property("Note").value = tostring(vb.views.Note.value)
      masking_table:property("Sign").value = tostring(vb.views.Sign.value)
      masking_table:property("Octave").value = tostring(vb.views.Octave.value)
      masking_table:property("Instrument").value = tostring(vb.views.Instrument.value)
      masking_table:property("Volume").value = tostring(vb.views.Volume.value)
      masking_table:property("Panning").value = tostring(vb.views.Panning.value)
      masking_table:property("Delay").value = tostring(vb.views.Delay.value)
    else
      vb.views.EffectNumber.value = false
      masking_table:property("EffectNumber").value = tostring(vb.views.EffectNumber.value)
      vb.views.EffectValue.value = false
      masking_table:property("EffectValue").value = tostring(vb.views.EffectValue.value)
      vb.views.Automation.value = false
      masking_table:property("Automation").value = tostring(vb.views.Automation.value)
    end

    masking_table:property("NoteMaskSelection").value = "1"
    masking_table:property("EffectMaskSelection").value = "1"
  end  

  masking_table:save_as(mask_file)
      
end


--------------------------------------------------------------------------------
-- GUI
--------------------------------------------------------------------------------


function masking_dialog()

  local CONTENT_MARGIN = renoise.ViewBuilder.DEFAULT_CONTROL_MARGIN
  local BUTTON_WIDTH = 20
  ok,err = masking_table:load_from(mask_file)

--[[
  if err then
    print("Error:"..err)
  else
    print("mask table loading status:"..tostring(ok))
  end
--]]
  
  local mask_objects = {
    "Note", "Sign", "Octave", "Instrument", "Volume", "Panning", "Delay", 
    "Effect Number", "Effect Value", "Automation"
  }

  if mask_dialog == nil then
    dialog_content = vb:column {
      margin = CONTENT_MARGIN
    }

    local mask_row = vb:row {}

    for mask = 1,#mask_objects do
      local loaded_state = mask_table[mask]

      if loaded_state == "true" then
        loaded_state = true
      else
        loaded_state = false
      end

      local mask_box = vb:column{
        vb:row{
          vb:text{
            text = mask_objects[mask],
          },
        },
        vb:row{
          vb:checkbox {
            width = BUTTON_WIDTH,
            id=mask_table[mask],
            value=loaded_state,
            notifier = function(value)
              masking_table[mask_table[mask]].value = tostring(value)
              masking_table:save_as(mask_file)
            end
          },
        },
      }

      mask_row:add_child(mask_box)
    end
    dialog_content:add_child(mask_row)
  

  end

    local function key_handler(dialog, key)

      if key.name == "comma" and key.modifiers == "shift" then
        mask_columns(MASK_SELECTED_INVERTED)
       end
      if key.name == "comma" and key.modifiers == "alt" then
        mask_columns(MASK_ALL_INVERTED)
      end    
      if key.name == "comma" and key.modifiers == "shift + alt" then
        mask_columns(MASK_ALL_TRUE)
      end    
      if key.name == "comma" and key.modifiers == "shift + control" then
        mask_columns(MASK_ALL_FALSE)
      end    
      if key.name == "comma" and key.modifiers == "shift + command" then
        mask_columns(MASK_ALL_FALSE)
      end    

      if (key.modifiers == "" and key.name == "esc") then
        dialog:close()
      end

    end  
    
  if mask_dialog == nil or not mask_dialog.visible then
    mask_dialog = renoise.app():show_custom_dialog(
      "Pattern Content Masking", dialog_content, key_handler)
  end
    
  vb.views.Note.value = toboolean(masking_table["Note"].value)
  vb.views.Sign.value = toboolean(masking_table["Sign"].value)
  vb.views.Octave.value = toboolean(masking_table["Octave"].value)
  vb.views.Instrument.value = toboolean(masking_table["Instrument"].value)
  vb.views.Volume.value = toboolean(masking_table["Volume"].value)
  vb.views.Panning.value = toboolean(masking_table["Panning"].value)
  vb.views.Delay.value = toboolean(masking_table["Delay"].value)
  vb.views.EffectNumber.value = toboolean(masking_table["EffectNumber"].value)
  vb.views.EffectValue.value = toboolean(masking_table["EffectValue"].value)
  vb.views.Automation.value = toboolean(masking_table["Automation"].value)
   
end

function toboolean(value)
  if string.lower(tostring(value)) == "false" or value == 0 or value == "0" then
    return false
  end
  
  return true
end


--------------------------------------------------------------------------------
-- Key Binding
--------------------------------------------------------------------------------

renoise.tool():add_keybinding {
  name = "Pattern Editor:Selection:Mask single note_effect column",
  invoke = function() mask_columns(MASK_SELECTED_INVERTED) end
}

renoise.tool():add_keybinding {
  name = "Pattern Editor:Selection:Invert all note/effect column maskings",
  invoke = function() mask_columns(MASK_ALL_INVERTED) end
}

renoise.tool():add_keybinding {
  name = "Pattern Editor:Selection:Mask all note/effect column",
  invoke = function() mask_columns(MASK_ALL_TRUE) end
}

renoise.tool():add_keybinding {
  name = "Pattern Editor:Selection:UnMask all note/effect column",
  invoke = function() mask_columns(MASK_ALL_FALSE) end
}


